@@ -1,4 +1,3 @@
-# $Id: Brick.pm 2260 2007-05-01 22:46:29Z comdog $
 package Brick;
 use strict;
 
@@ -10,8 +9,7 @@ use Data::Dumper;
 
 use Brick::Profile;
 
-$VERSION = '0.223';
-	#sprintf "0.%04d_01", q$Revision: 2260 $ =~ m/(\d+)/g;
+$VERSION = '0.227';
 
 =head1 NAME
 
@@ -38,7 +36,7 @@ Brick - Complex business rule data validation
 		name => 'Joe Snuffy',
 		...
 		);
-		
+
 	my $results = $brick->apply( $profile, \%%input_from_app );
 
 =head1 DESCRIPTION
@@ -182,7 +180,7 @@ sub create_bucket
 
 		$args->{code} = do {
 			if( eval { $method->isa( ref {} ) } or
-				UNIVERSAL::isa( $method, ref sub {} ) )
+				ref $method eq ref sub {} )
 				{
 				$method;
 				}
@@ -217,12 +215,12 @@ sub init
 
 	$self->{buckets} = [];
 
-	if( defined $args->{external_packages} && UNIVERSAL::isa( $args->{external_packages}, ref [] ) )
+	if( defined $args->{external_packages} && ref $args->{external_packages} eq ref [] )
 		{ # defined and array ref
 		$self->{external_packages} = $args->{external_packages};
 		}
 	elsif( defined $args->{external_packages} &&
-		! UNIVERSAL::isa( $args->{external_packages}, ref [] ) )
+		! ($args->{external_packages} eq ref []) )
 		{ # defined but not array ref
 		carp "'external_packages' value must be an anonymous array";
 		$self->{external_packages} = [];
@@ -268,92 +266,12 @@ sub clone
 	$brick;
 	}
 
-=item explain( PROFILE || PROFILE_ARRAYREF )
-
-Turn the profile into a textual description without applying it to any
-data. This does not add the profile to instance and it does not add
-the constraints to the bucket.
-
-If everything goes right, this returns a single string that represents
-the profile.
-
-If the profile does not pass the C<lint> test, this returns undef or the
-empty list.
-
-If you want to do something with a datastructure, you probably want to
-write a different method very similar to this instead of trying to parse
-the output.
-
-Future notes: maybe this is just really a dispatcher to things that do
-it in different ways (text output, hash output).
-
-=cut
-
 sub explain
 	{
-	my( $brick, $profile_or_array ) = @_;
-
-	my $profile = do {
-		unless( eval { $profile_or_array->isa( $brick->profile_class ) } )
-			{
-			use Brick::Profile;
-			
-			eval "require " . $brick->profile_class;
-			
-			my $profile = $brick->profile_class->new( $brick, $profile_or_array );
-
-			return unless eval { $profile->isa( $brick->profile_class ) };
-			
-			$profile;
-			}
-		else
-			{
-			$profile_or_array;
-			}
-		};
-
-	my $bucket   = $profile->get_bucket;
-	my $coderefs = $profile->get_coderefs;
-	my $array    = $profile->get_array;
-
-	my @entries = map {
-		my $e = $bucket->get_from_bucket( $_ );
-		[ map { $e->$_ } qw(get_coderef get_name) ]
-		} @$coderefs;
-
-	#print STDERR Data::Dumper->Dump( [ \@entries ], [qw(entries)] );
-
-	my $level = 0;
-	my $str   = '';
-	foreach my $index ( 0 .. $#entries )
-		{
-		my $tuple = $entries[$index];
-
-		my @uses = ( [ $level, $tuple->[0] ] );
-
-		#print STDERR Data::Dumper->Dump( [ \@uses ], [qw(uses)] );
-
-		while( my $pair = shift @uses )
-			{
-			my $entry = $bucket->get_from_bucket( $pair->[1] );
-			#print Data::Dumper->Dump( [ $entry ], [qw(entry)] );
-			next unless $entry;
-
-			$str .=  "\t" x $pair->[0] . $entry->get_name . "\n";
-
-			unshift @uses, map {
-				[ $pair->[0] + 1, $_ ]
-				} @{ $entry->get_comprises( $pair->[1] ) };
-			#print Data::Dumper->Dump( [ \@uses ], [qw(uses)] );
-			}
-
-		$str.= "\n";
-		}
-
-	$str;
+	croak "Who's calling Brick::explain? That's in Brick::Profile now!";
 	}
-	
-=item apply(  PROFILE_ARRAYREF | PROFILE OBJECT, INPUT_DATA_HASHREF )
+
+=item apply(  PROFILE OBJECT, INPUT_DATA_HASHREF )
 
 Apply the profile to the data in the input hash reference. The profile
 can either be a profile object or an array ref that apply() will use to
@@ -367,31 +285,15 @@ that, you can override it in your own subclass.
 
 sub apply
 	{
-	my( $brick, $profile_or_array, $input ) = @_;
+	my( $brick, $profile, $input ) = @_;
+
+	croak "Did not get a profile object in Brick::apply()!\n"
+		unless eval { $profile->isa( $brick->profile_class ) };
 
-	my $profile = do {
-		unless( eval { $profile_or_array->isa( $brick->profile_class ) } )
-			{
-			use Brick::Profile;
-			
-			eval "require " . $brick->profile_class;
-			
-			my $profile = $brick->profile_class->new( $brick, $profile_or_array );
-						
-			return unless eval { $profile->isa( $brick->profile_class ) };
-			
-			$profile;
-			}
-		else
-			{
-			$profile_or_array;
-			}
-		};
-	
 	my $bucket   = $profile->get_bucket;
 	my $coderefs = $profile->get_coderefs;
 	my $array    = $profile->get_array;
-	
+
 	my @entries = map {
 		my $e = $bucket->get_from_bucket( $_ );
 		[ map { $e->$_ } qw(get_coderef get_name) ]
@@ -404,29 +306,49 @@ sub apply
 		my $e    = $entries[$index];
 		my $name = $array->[$index][0];
 
-		print STDERR "Checking $name...\n" if $ENV{DEBUG};
+		my $bucket_entry = $bucket->get_from_bucket( "$e->[0]" );
+		my $sub_name     = $bucket_entry->get_name;
 
 		my $result = eval{ $e->[0]->( $input ) };
 		my $eval_error = $@;
 
-		#print STDERR Data::Dumper->Dump( [$eval_error], [qw(eval_error)] );
+		carp "Brick: $sub_name: eval error \$\@ is not a string or hash reference"
+			unless( ! ref $eval_error or ref $eval_error eq ref {} );
 
-		$result = 0 if ref $eval_error;
+		if( defined $eval_error and ref $eval_error eq ref {} )
+			{
+			$result = 0;
+			carp "Brick: $sub_name died with reference, but didn't define 'handler' key"
+				unless exists $eval_error->{handler};
+
+			carp "Brick: $sub_name died with reference, but didn't define 'message' key"
+				unless exists $eval_error->{message};
+			}
+		elsif( defined $eval_error ) # but not a reference
+			{
+			$eval_error = {
+				handler       => 'program_error',
+				message       => $eval_error,
+				program_error => 1,
+				errors        => [],
+				};
+			}
 
 		my $handler = $array->[$index][1];
 
-		push @results, [ $name, $handler, $result, $@ ];
+		my $result_item = $brick->result_class->result_item_class->new(
+			label    => $name,
+			method   => $handler,
+			result   => $result,
+			messages => $eval_error,
+			);
+
+		push @results, $result_item;
 		}
 
-	eval { eval "use " . $brick->result_class };
-	if( $@ ) 
-		{ 
-		carp "Couldn not load Result class " . $brick->result_class . "\n" 
-		}
-	
 	return bless \@results, $brick->result_class;
 	}
-		
+
 =item bucket_class
 
 The namespace where the constraint building blocks are defined. By
@@ -435,9 +357,12 @@ this in a subclass. Things that need to work with the bucket class
 name, such as a factory method, will use the return value of this
 method.
 
+This method also loads the right class, so if you override it,
+remember to load the class too!
+
 =cut
 
-sub bucket_class { 'Brick::Bucket' }
+sub bucket_class { require Brick::Bucket; 'Brick::Bucket' }
 
 =item result_class
 
@@ -446,9 +371,12 @@ this is C<Brick::Result>. If you don't like that, override this in a
 subclass. Things that need to work with the result class name, such as
 a factory method, will use the return value of this method.
 
+This method also loads the right class, so if you override it,
+remember to load the class too!
+
 =cut
 
-sub result_class { 'Brick::Result' }
+sub result_class { require Brick::Result; 'Brick::Result' }
 
 =item profile_class
 
@@ -457,6 +385,9 @@ C<Brick::Profile>. If you don't like that, override this in a
 subclass. Things that need to work with the result class name, such as
 a factory method, will use the return value of this method.
 
+This method also loads the right class, so if you override it,
+remember to load the class too!
+
 =cut
 
 sub profile_class { require Brick::Profile; 'Brick::Profile' }
@@ -473,13 +404,9 @@ L<Brick::Tutorial>, L<Brick::UserGuide>
 
 =head1 SOURCE AVAILABILITY
 
-This source is part of a SourceForge project which always has the
-latest sources in SVN, as well as all of the previous releases.
+This source is in Github:
 
-	svn co https://brian-d-foy.svn.sourceforge.net/svnroot/brian-d-foy brian-d-foy
-
-If, for some reason, I disappear from the world, one of the other
-members of the project can shepherd this module appropriately.
+	https://github.com/briandfoy/brick
 
 =head1 AUTHOR
 
@@ -487,7 +414,7 @@ brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2007, brian d foy, All Rights Reserved.
+Copyright (c) 2007-2014, brian d foy, All Rights Reserved.
 
 You may redistribute this under the same terms as Perl itself.
 
